import * as React from 'react';
import { Table, Button, Header, Container, Menu, Icon } from 'semantic-ui-react';
import { ListContainer } from '@overflow/commons/react/component/ListContainer';
import Probe from '@overflow/probe/api/model/Probe';
import Domain from '@overflow/domain/api/model/Domain';
import History from '@overflow/history/api/model/History';
import Page from '@overflow/commons/api/model/Page';
import MetaHistoryType from '@overflow/meta/api/model/MetaHistoryType';
import { Pager } from '@overflow/commons/react/component/Pager';


export interface StateProps {
  probes: Probe[];
  histories: Page;
  path: any;
}

export interface DispatchProps {
  onReadAllProbeByDomain?(domain: Domain): void;

  onReadAllByDomain?(domain: Domain, pageNo: string, countPerPage: string): void;
  onReadAllByProbe?(probe: Probe, pageNo: string, countPerPage: string): void;
  onReadAllByProbeAndType?(probe: Probe, type: MetaHistoryType, pageNo: string, countPerPage: string): void;
}

export type Props = StateProps & DispatchProps;

export interface State {
  isTypedHistory: boolean;
  selectedProbeId: number;
}

export class HistoryList extends React.Component<Props, State> {

  private countPerPage: number = 10;

  constructor(props: Props, context: State) {
    super(props, context);
    this.state = {
      isTypedHistory: false,
      selectedProbeId: 0,
    };
  }

  public componentWillMount(): void {
    let domain: Domain = {
      id: 1,
    };
    this.props.onReadAllProbeByDomain(domain);
    if (this.props.path === '/histories') {
      this.setState({
        isTypedHistory: false,
      });
      this.getAllHistory(0);
    } else {
      this.setState({
        isTypedHistory: true,
      });
      this.getTypedHistory(0);
    }
  }

  public getAllHistory(pageNo: number): void {
    if (this.state.selectedProbeId === 0) {
      let domain: Domain = {
        id: Number(1),
      };
      this.props.onReadAllByDomain(domain, String(pageNo), String(this.countPerPage));
    } else {
      let probe: Probe = {
        id: Number(this.state.selectedProbeId),
      };
      this.props.onReadAllByProbe(probe, String(pageNo), String(this.countPerPage));
    }
  }

  public getTypedHistory(pageNo: number): void {
    let probe: Probe = {
      id: Number(1),
    };
    let type = this.props.path.split('/')[1];
    this.props.onReadAllByProbeAndType(probe, this.getMetaType(type), String(pageNo), String(this.countPerPage));
  }

  public getMetaType(typeStr: string): MetaHistoryType {
    let metaType: MetaHistoryType = {
    };
    if (typeStr === 'member') {
      metaType.id = 1;
    } else if (typeStr === 'probe') {
      metaType.id = 2;
    } else if (typeStr === 'discovery') {
      metaType.id = 3;
    } else if (typeStr === 'target') {
      metaType.id = 4;
    } else if (typeStr === 'crawler') {
      metaType.id = 5;
    } else if (typeStr === 'sensor') {
      metaType.id = 6;
    }
    return metaType;
  }

  public handlePaging = (pageNo: number) => {
    let probe: Probe = {
      id: Number(1),
    };
    if (this.state.isTypedHistory) {
      this.getTypedHistory(pageNo);
    } else {
      this.getAllHistory(pageNo);
    }
  }

  public render(): JSX.Element {
    if (this.props.probes !== undefined) {
      console.log(this.props.probes);
    }
    let historyList: JSX.Element = (
      <Container fluid>
        <Table celled selectable striped>
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell textAlign={'center'}>No.</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Type</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Message</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Created At</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Created By</Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {this.props.histories && this.props.histories.content ?
              this.props.histories.content.map((history: History, index: number) => (
                <Table.Row key={index} >
                  <Table.Cell textAlign={'center'}>{history.id}</Table.Cell>
                  <Table.Cell textAlign={'center'}>
                    {history.type.name}
                  </Table.Cell>
                  <Table.Cell>
                    {history.message}
                  </Table.Cell>
                  <Table.Cell>
                    {history.createDate}
                  </Table.Cell>
                  <Table.Cell>
                    {history.member.name}
                  </Table.Cell>
                </Table.Row>
              )) : ''}
          </Table.Body>
          <Table.Footer>
            <Table.Row>
              <Table.HeaderCell colSpan='3'>
                <Pager page={this.props.histories} onPageChange={this.handlePaging} />
              </Table.HeaderCell>
            </Table.Row>
          </Table.Footer>
        </Table>
      </Container>
    );

    return (
      <ListContainer
        contents={historyList}
        data={this.props.histories}
      />
    );
  }
}


