import * as React from 'react';
import {
  Icon,
  Step,
  Button,
  Table,
  Radio,
  Form,
  Container,
  Checkbox,
} from 'semantic-ui-react';
import {
  Grid,
  Image,
  Label,
  Segment,
  Dropdown,
  Input,
  List,
  Accordion,
  Loader,
  DropdownItemProps,
} from 'semantic-ui-react';

import MetaCrawler from '@overflow/meta/api/model/MetaCrawler';
import Infra from '@overflow/infra/api/model/Infra';
import Domain from '@overflow/domain/api/model/Domain';

import SensorItemTree from '@overflow/meta/react/SensorItemTree';
import CrawlerSelectorContainer from '@overflow/meta/react/CrawlerSelector';

export interface SensorConfigurationStateProps {
  infraId?: number;
  infraList?: Infra[];
  infra?: Infra;
}

export interface SensorConfigurationDispatchProps {
  onReadAllTargetByDomain?(domain: Domain): void;
  onReadInfra?(infraId: number): void;
}

export interface SensorConfigurationState {
  selectedCrawlerId: number;
}

export type SensorConfigurationProps = SensorConfigurationStateProps & SensorConfigurationDispatchProps;

export class SensorConfiguration extends React.Component<SensorConfigurationProps, SensorConfigurationState> {

  constructor(props: SensorConfigurationProps, context: SensorConfigurationState) {
    super(props, context);
    this.state = {
      selectedCrawlerId: 0,
    };
  }

  public componentWillMount(): void {
    if (this.props.infraId === undefined) {
      this.props.onReadAllTargetByDomain({ id: 1 });
    } else {
      this.props.onReadInfra(this.props.infraId);
    }
  }

  public onSelectCrawlerId = (id: number) => {
    this.setState({
      selectedCrawlerId: id,
    });
  }


  public render(): JSX.Element {
    let steps = [<CrawlerSelectorContainer onSelectCrawlerId={this.onSelectCrawlerId.bind(this)} />,
    <Segment vertical><SensorItemTree crawlerId={this.state.selectedCrawlerId} /></Segment>, <ETCSelector />];

    return (
      <ConfigStepper steps={steps} infra={this.props.infra} infraList={this.props.infraList} />
    );
  }
}


export interface ConfigStepperProps {
  steps: Array<JSX.Element>;
  infraList?: Infra[];
  infra?: Infra;
}

export interface ConfigStepperState {
  currentStep: number;
}

export class ConfigStepper extends React.Component<ConfigStepperProps, ConfigStepperState> {

  private selectOptions: Array<DropdownItemProps>;

  constructor(props: ConfigStepperProps, context: ConfigStepperState) {
    super(props, context);
    this.state = {
      currentStep: 1,
    };
  }

  public convertInfraList(): void {

    if (this.props.infraList === undefined) {
      return null;
    }

    let selectionOptions: Array<DropdownItemProps> = new Array;

    for (let infra of this.props.infraList) {

      let option = {
        key: infra.target.id,
        text: infra.target.displayName,
        value: infra.target.id,
        icon: 'check', // or close?
      };
      selectionOptions.push(option);

    }
    this.selectOptions = selectionOptions;
  }

  public handleActive(idx: number): boolean {
    if (this.state.currentStep === idx) {
      return true;
    }
    return false;
  }

  public handleCompleted(idx: number): boolean {
    if (this.state.currentStep > idx) {
      return true;
    }
    return false;
  }

  public handlePrev(event: React.SyntheticEvent<HTMLAnchorElement>, data: object): void {
    let step: number = this.state.currentStep;
    if (step <= 1) {
      step = 1;
    } else {
      step--;
    }
    this.setState({
      currentStep: step,
    });
  }
  public handleNext(event: React.SyntheticEvent<HTMLAnchorElement>, data: object): void {
    let step: number = this.state.currentStep;
    if (step >= this.props.steps.length + 1) {
      step = this.props.steps.length + 1;
    } else {
      step++;
    }
    this.setState({
      currentStep: step,
    });
    if (step === this.props.steps.length + 1) {
      console.log('Done');
    }
  }

  public showContent(): any {
    return this.props.steps[this.state.currentStep - 1];
  }

  public checkPrevDisabled(): boolean {
    if (this.state.currentStep <= 1) {
      return true;
    } else {
      return false;
    }
  }

  public checkNextDisabled(): boolean {
    if (this.state.currentStep === this.props.steps.length + 1) {
      return true;
    } else {
      return false;
    }
  }

  public renderInfraList(): JSX.Element {

    if (this.props.infraList === undefined) {
      return null;
    }

    this.convertInfraList();

    // let elems: Array<JSX.Element> = new Array;

    // elems.push(
    return (<Dropdown openOnFocus
      placeholder='Select Target' selection options={this.selectOptions} />);
    // );

    // return elems;
  }

  public render(): JSX.Element {

    return (
      <Container fluid>
        <Step.Group fluid>
          <Step active={this.handleActive(1)} completed={this.handleCompleted(1)}>
            <Icon name='settings' color='grey' />
            <Step.Content title='Step 1' description='골라골라 크롤러를 골라' />
          </Step>

          <Step active={this.handleActive(2)} completed={this.handleCompleted(2)}>
            <Icon name='list ul' color='grey' />
            <Step.Content title='Step 2' description='골라골라 아이템도 골라' />
          </Step>

          <Step active={this.handleActive(3)} completed={this.handleCompleted(3)}>
            <Icon name='options' color='grey' />
            <Step.Content title='Step 3' description='나머지도 골라 아 빨리' />
          </Step>
        </Step.Group>
        {this.renderInfraList()}
        {this.showContent()}
        <br />
        <Button.Group floated='right'> {/* floated 사용시 레이아웃 깨지는 현상 */}
          <Button content='Prev' icon='left arrow' labelPosition='left'
            onClick={this.handlePrev.bind(this)} disabled={this.checkPrevDisabled()} />
          <Button.Or />
          <Button content='Next' icon='right arrow' labelPosition='right' positive
            onClick={this.handleNext.bind(this)} disabled={this.checkNextDisabled()} />
        </Button.Group>
      </Container>
    );
  }
}



export interface ETCSelectorProps {

}

export interface ETCSelectorState {
  interval: number;
}

export class ETCSelector extends React.Component<ETCSelectorProps, ETCSelectorState> {
  constructor(props: any, context: any) {
    super(props, context);
    this.state = {
      interval: 5,
    };
  }
  public handleIntervalChange(e: any, { value }: any): void {
    this.setState({ interval: value });
  }

  public render(): JSX.Element {
    return (
      <Table celled={true}>
        <Table.Body>
          <Table.Row>
            <Table.Cell collapsing>
              Interval
            </Table.Cell>
            <Table.Cell>
              <Form.Group inline>
                <Form.Radio
                  label='5 seconds'
                  name='radioGroup'
                  value='5'
                  checked={this.state.interval === 5}
                  onChange={this.handleIntervalChange}
                />
                <Form.Radio
                  label='10 seconds'
                  name='radioGroup'
                  value='10'
                  checked={this.state.interval === 10}
                  onChange={this.handleIntervalChange}
                />
                <Form.Radio
                  label='15 seconds'
                  name='radioGroup'
                  value='15'
                  checked={this.state.interval === 15}
                  onChange={this.handleIntervalChange}
                />
              </Form.Group>
            </Table.Cell>
          </Table.Row>
          <Table.Row>
            <Table.Cell collapsing>
              Notification
            </Table.Cell>
            <Table.Cell></Table.Cell>
          </Table.Row>
          <Table.Row>
            <Table.Cell collapsing>
              Warn Condition
            </Table.Cell>
            <Table.Cell></Table.Cell>
          </Table.Row>
        </Table.Body>
      </Table>
    );
  }
}
